from io import BytesIO

from httpx import AsyncClient
from PIL import Image, ImageDraw, ImageFont

from .model import VideoInfo
import time


def circle_corner(img, radii):  # 把原图片变成圆角，
    """
    圆角处理
    :param img: 源图象。
    :param radii: 半径，如：30。
    :return: 返回一个圆角处理后的图象。
    """

    # 画圆（用于分离4个角）
    circle = Image.new("L", (radii * 2, radii * 2), 0)  # 创建一个黑色背景的画布
    draw = ImageDraw.Draw(circle)
    draw.ellipse((0, 0, radii * 2, radii * 2), fill=255)  # 画白色圆形

    # 原图
    img = img.convert("RGBA")
    w, h = img.size

    # 画4个角（将整圆分离为4个部分）
    alpha = Image.new("L", img.size, 255)
    alpha.paste(circle.crop((0, 0, radii, radii)), (0, 0))  # 左上角
    alpha.paste(circle.crop((radii, 0, radii * 2, radii)), (w - radii, 0))  # 右上角
    alpha.paste(
        circle.crop((radii, radii, radii * 2, radii * 2)), (w - radii, h - radii)
    )  # 右下角
    alpha.paste(circle.crop((0, radii, radii, radii * 2)), (0, h - radii))  # 左下角
    # alpha.show()

    img.putalpha(alpha)  # 白色区域透明可见，黑色区域不可见
    return img


async def build_get_share_info(video_info: VideoInfo) -> bytes:
    # 图片尺寸 1750 x 1500
    img_width = 1750
    img_height = 1500
    # 创建一个空白图片，背景色为白色
    img = Image.new("RGB", (img_width, img_height), (255, 255, 255))
    # 绘制UP主信息
    # UP头像贴到左上角
    async with AsyncClient() as client:
        resp = await client.get(video_info.upInfo.iconUrl)
        img_up_icon = Image.open(BytesIO(resp.content))
        img_up_icon = img_up_icon.resize((150, 150))
        img.paste(img_up_icon, (25, 25))
    # UP名字绘制到头像右侧，间距为25
    draw = ImageDraw.Draw(img)
    font = ImageFont.truetype("msyh.ttc", 60)
    draw.text(
        (img_up_icon.width + 50, 30), video_info.upInfo.name, (251, 114, 153), font
    )
    # 发布时间绘制到UP名字下方，间距为25
    font = ImageFont.truetype("msyh.ttc", 80)
    # 计算发布时间
    now_time = int(time.time())
    time_diff = now_time - video_info.pubdate
    if time_diff < 60:
        time_str = "刚刚"
    elif time_diff < 3600:
        time_str = f"{int(time_diff / 60)}分钟前"
    elif time_diff < 3600 * 24:
        hours = int(time_diff / 3600 % 24)
        time_str = f"{hours}小时前"
    else:
        days = int(time_diff / 3600 / 24)
        time_str = f"{days}天前"
    # 绘制发布时间,字体颜色为灰色
    font = ImageFont.truetype("msyh.ttc", 50)
    draw.text(
        (img_up_icon.width + 50, 105),
        time_str,
        (128, 128, 128),
        font,
    )
    # 将网络图片转换为PIL图片
    async with AsyncClient() as client:
        resp = await client.get(video_info.pic)
        if resp.status_code == 200:
            picture = Image.open(BytesIO(resp.content))
            # 将图片缩放到宽度与空白图片左右留白25像素一致，高度自适应
            picture = picture.resize(
                (img_width - 50, int(picture.height * (img_width - 50) / picture.width))
            )
            # 将图片绘制到空白图片上
            img.paste(picture, (25, 300))

    # 绘制视频信息
    # 视频标题绘制到图片上，左边距为25
    title = video_info.title
    font = ImageFont.truetype("msyh.ttc", 90)
    if img_width - 50 < font.getsize(title)[0]:
        sum = 0
        for i in range(len(title)):
            sum += font.getsize(title[i])[0]
            if sum > img_width - 50:
                title = title[: i - 1] + "···"
                break
    # 在空白图片绘制文字,上边距为190，左边距为25，字体大小为60，字体颜色为黑
    draw.text((25, 180), title, (0, 0, 0), font=font)
    # 绘制视频简介
    desc = video_info.desc.replace("\n", "")
    font = ImageFont.truetype("msyh.ttc", 50)
    _height = 0
    # 如果视频简介超出2行
    if font.getsize(desc)[0] > (img_width - 50) * 2:
        # 计算简介字符的超出位数
        sum = 0
        n = 0
        for i in range(len(desc)):
            sum += font.getsize(desc[i])[0]
            if sum > (img_width - 50) * 2:
                # 将简介字符超出的部分删除
                desc = desc[: i - 5] + "···"
                break
            if sum > (img_width - 50):
                if n == 0:
                    n = i
        _height = img_height - font.getsize(desc)[1] * 2 - 5
        for i in range(0, len(desc), n):
            draw.text((25, _height), desc[i : i + n], (0, 0, 0), font=font)
            _height += font.getsize(desc[i : i + n])[1]
    # 如果简介不足2行，则图片高度裁剪70像素
    elif font.getsize(desc)[0] < (img_width - 50):
        img = img.crop((0, 0, img_width, img_height - 70))
        img_height -= 70
        _height = img_height - font.getsize(desc)[1] - 5
        draw = ImageDraw.Draw(img)
        # 在空白图片底部绘制文字,下边距为font高度乘2加25，左边距为25，字体颜色为黑
        draw.text((25, _height), desc, (0, 0, 0), font=font)

    # 将图片转换为PIL图片
    img = img.convert("RGB")
    # 将PIL图片转换为bytes
    img_bytes = BytesIO()
    img.save(img_bytes, "png")
    return img_bytes.getvalue()




from nonebot.adapters.onebot.v11 import MessageEvent, MessageSegment
from nonebot import on_message
from nonebot_plugin_guild_patch import GuildMessageEvent
from typing import Union
from .utils import *
from .img import build_get_share_info

share_sort_url = on_message(block=False)


@share_sort_url.handle()
async def _(event: Union[MessageEvent, GuildMessageEvent]):
    if event.raw_message.startswith("av") and event.raw_message[2:].isdigit():
        video_id = event.raw_message[2:]
    elif event.raw_message.startswith("BV"):
        video_id = await bv_to_av(event.raw_message)
    else:
        return
    viode_info = await get_video_info(video_id)
    if viode_info:
        msg = MessageSegment.image(
            await build_get_share_info(viode_info)
        ) + MessageSegment.text("\n点击前往:" + viode_info.shareUrl)
        await share_sort_url.finish(msg)

__version__ = '0.1.0'



# UP主信息类
class UpInfo:
    def __init__(self, uid: int, name: str, iconUrl: str):
        self.uid = uid
        self.name = name
        self.iconUrl = iconUrl


# 视频信息类
class VideoInfo:
    def __init__(
        self,
        title: str,
        pictureUrl: str,
        desc: str,
        pubdate: int,
        upInfo: UpInfo,
        shareUrl: str,
    ) -> None:
        self.title = title
        self.pic = pictureUrl
        self.desc = desc
        self.pubdate = pubdate
        self.upInfo = upInfo
        self.shareUrl = shareUrl



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_bilibili_viode']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=9.1.1,<10.0.0',
 'httpx>=0.23.0,<0.24.0',
 'nonebot-plugin-guild-patch>=0.1.3,<0.2.0']

setup_kwargs = {
    'name': 'nonebot-plugin-bilibili-viode',
    'version': '0.1.0',
    'description': '一个nonebot2插件，用于获取哔哩哔哩伪分享卡片',
    'long_description': '# nonebot_plugin_bilibili_viode\n\nnonebot_plugin_bilibili_viode是一个Nonebot2的插件，其功能为将用户发送的B站视频ID转为(伪)分享卡片的形式  \n\n## 如何安装使用\n### 安装\n`pip insall nonebot_plugin_bilibili_viode`  \n或者  \n`poetry add nonebot_plugin_bilibili_viode`  \n### 使用\n在你的nontbot项目中的bot.py文件中添加  \n`nonebot.load_plugin("nonebot_plugin_bilibili_viode")`\n',
    'author': 'ASTWY',
    'author_email': 'astwy@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/nonebot_plugin_bilibili_viode/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from httpx import AsyncClient
from .model import *

# BV号转AV号
async def bv_to_av(bv_id: str) -> str:
    async with AsyncClient() as client:
        resp = await client.get(
            f"https://api.bilibili.com/x/web-interface/view?bvid={bv_id}"
        )
        if resp.status_code == 200:
            data = resp.json()
            if data["code"] == 0:
                return str(data["data"]["aid"])


# 获取B站视频的短链接
async def get_share_sort_url(av_id: str):
    body = {
        "build": "6060600",
        "buvid": "0",
        "oid": av_id,
        "platform": "android",
        "share_channel": "QQ",
        "share_id": "main.ugc-video-detail.0.0.pv",
        "share_mode": "7",
    }
    async with AsyncClient() as client:
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        resp = await client.post(
            url="http://api.biliapi.net/x/share/click", headers=headers, data=body
        )
        if resp.status_code == 200:
            data = resp.json()
            if data["code"] == 0:
                return data["data"]["link"]
    return None


# 获取B站视频的视频基本信息
# 传入参数：AV号或者BV号
# 返回值：视频信息类
async def get_video_info(av_id: str):
    if av_id.startswith("BV"):
        av_id = await bv_to_av(av_id)
    shareUrl = await get_share_sort_url(av_id)
    apiUrl = "http://api.bilibili.com/x/web-interface/view?aid=" + av_id
    async with AsyncClient() as client:
        resp = await client.get(apiUrl)
        if resp.status_code == 200:
            data = resp.json()
            if data["code"] == 0:
                up_info = UpInfo(
                    uid=data["data"]["owner"]["mid"],
                    name=data["data"]["owner"]["name"],
                    iconUrl=data["data"]["owner"]["face"],
                )
                return VideoInfo(
                    title=data["data"]["title"],
                    pictureUrl=data["data"]["pic"],
                    desc=data["data"]["desc"],
                    pubdate=data["data"]["pubdate"],
                    upInfo=up_info,
                    shareUrl=shareUrl
                    if shareUrl
                    else "http://www.bilibili.com/video/av" + av_id,
                )
    return None

